/****************************************************************************
 *
 *   File: boolean_expr.cc:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Aug  6 12:19:06 2002
 *
 *   Function: A function that returns a boolean value.
 *
 ***************************************************************************/

#include "boolean_expr.h"


boolean_expr::boolean_expr (genetic_expr_bnf* bnf, 
			    genetic_expr* parent,
			    uint32 num_rands) :
///////////////////////////////////////////
//////// booloan_expr stuff. //////////////
  genetic_expr (bnf, parent, num_rands)
{
}

expr_lt::expr_lt (genetic_expr_bnf* bnf, genetic_expr* parent) :
///////////////////////////////////////////
////////////// expr < expr ////////////////
  boolean_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_lt::random (genetic_expr_bnf* bnf, 
		 uint32 h,
		 genetic_expr* parent)
{
  expr_lt *lt = new expr_lt (bnf, parent);
  lt->rands()[0] = genetic_expr::random (bnf, h, lt);
  lt->rands()[1] = genetic_expr::random (bnf, h, lt);
  return lt;
}

genetic_expr*
expr_lt::parse_expr (genetic_expr_bnf* bnf, 
		     genetic_expr* parent, 
		     genetic_lexer* lexer)
{
  expr_lt *lt = new expr_lt (bnf, parent);
  lt->rands()[0] = genetic_expr::parse_dbl_expr (bnf, lt, lexer);
  lt->rands()[1] = genetic_expr::parse_dbl_expr (bnf, lt, lexer);
  return lt;
}

lval
expr_lt::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  tmp.i = (rands()[0]->evaluate(dbls, bools, sample).d < 
	   rands()[1]->evaluate(dbls, bools, sample).d);
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_lt::clone ()
{
  expr_lt *tmp = new expr_lt (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}


expr_gt::expr_gt (genetic_expr_bnf* bnf, genetic_expr* parent) :
///////////////////////////////////////////
////////////// expr > expr ////////////////
  boolean_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_gt::random (genetic_expr_bnf* bnf, 
		 uint32 h,
		 genetic_expr* parent)
{
  expr_gt *gt = new expr_gt (bnf, parent);
  gt->rands()[0] = genetic_expr::random (bnf, h, gt);
  gt->rands()[1] = genetic_expr::random (bnf, h, gt);
  return gt;
}

genetic_expr*
expr_gt::parse_expr (genetic_expr_bnf* bnf, 
		     genetic_expr* parent, 
		     genetic_lexer* lexer)
{
  expr_gt *gt = new expr_gt (bnf, parent);
  gt->rands()[0] = genetic_expr::parse_dbl_expr (bnf, gt, lexer);
  gt->rands()[1] = genetic_expr::parse_dbl_expr (bnf, gt, lexer);
  return gt;
}

lval
expr_gt::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;

  tmp.i = (rands()[0]->evaluate(dbls, bools, sample).d > 
	   rands()[1]->evaluate(dbls, bools, sample).d);
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_gt::clone ()
{
  expr_gt *tmp = new expr_gt (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}


expr_eq::expr_eq (genetic_expr_bnf* bnf, 
		  genetic_expr* parent) :
///////////////////////////////////////////
////////////// expr == expr ///////////////
  boolean_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_eq::random (genetic_expr_bnf* bnf, 
		 uint32 h,
		 genetic_expr* parent)
{
  expr_eq *eq = new expr_eq (bnf, parent);
  eq->rands()[0] = genetic_expr::random (bnf, h, eq);
  eq->rands()[1] = genetic_expr::random (bnf, h, eq);
  return eq;
}

genetic_expr*
expr_eq::parse_expr (genetic_expr_bnf* bnf, 
		     genetic_expr* parent, 
		     genetic_lexer* lexer)
{
  expr_eq *eq = new expr_eq (bnf, parent);
  eq->rands()[0] = genetic_expr::parse_dbl_expr (bnf, eq, lexer);
  eq->rands()[1] = genetic_expr::parse_dbl_expr (bnf, eq, lexer);
  return eq;
}

lval
expr_eq::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;

  tmp.i = (rands()[0]->evaluate(dbls, bools, sample).d == 
	   rands()[1]->evaluate(dbls, bools, sample).d);
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_eq::clone ()
{
  expr_eq *tmp = new expr_eq (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}


expr_and::expr_and (genetic_expr_bnf* bnf, 
		    genetic_expr* parent) :
///////////////////////////////////////////
///////////// expr && expr ////////////////
  boolean_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_and::random (genetic_expr_bnf* bnf, 
		  uint32 h,
		  genetic_expr* parent)
{
  expr_and *_and = new expr_and (bnf, parent);
  _and->rands()[0] = genetic_expr::random (bnf, h, _and, BOOLEAN);
  _and->rands()[1] = genetic_expr::random (bnf, h, _and, BOOLEAN);
  return _and;
}

genetic_expr*
expr_and::parse_expr (genetic_expr_bnf* bnf, 
		      genetic_expr* parent, 
		      genetic_lexer* lexer)
{
  expr_and *_and = new expr_and (bnf, parent);
  _and->rands()[0] = genetic_expr::parse_bool_expr (bnf, _and, lexer);
  _and->rands()[1] = genetic_expr::parse_bool_expr (bnf, _and, lexer);
  return _and;
}

lval
expr_and::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;

  if (sample) 
  {
    int32 r = rands()[1]->evaluate(dbls, bools, sample).i;
    tmp.i = (rands()[0]->evaluate(dbls, bools, sample).i && r);
    record (tmp, sample);
  }
  else 
  {
    tmp.i = (rands()[0]->evaluate(dbls, bools, sample).i &&
	     rands()[1]->evaluate(dbls, bools, sample).i);
  }
  return tmp;
}

genetic_expr* 
expr_and::clone ()
{
  expr_and *tmp = new expr_and (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_or::expr_or (genetic_expr_bnf* bnf, 
		  genetic_expr* parent) :
///////////////////////////////////////////
///////////// expr || expr ////////////////
  boolean_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_or::random (genetic_expr_bnf* bnf, 
		 uint32 h,
		 genetic_expr* parent)
{
  expr_or *_or = new expr_or (bnf, parent);
  _or->rands()[0] = genetic_expr::random (bnf, h, _or, BOOLEAN);
  _or->rands()[1] = genetic_expr::random (bnf, h, _or, BOOLEAN);
  return _or;
}

genetic_expr*
expr_or::parse_expr (genetic_expr_bnf* bnf, 
		     genetic_expr* parent, 
		     genetic_lexer* lexer)
{
  expr_or *_or = new expr_or (bnf, parent);
  _or->rands()[0] = genetic_expr::parse_bool_expr (bnf, _or, lexer);
  _or->rands()[1] = genetic_expr::parse_bool_expr (bnf, _or, lexer);
  return _or;
}

lval
expr_or::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;

  if (sample) 
  {
    int32 r = rands()[1]->evaluate(dbls, bools, sample).i;
    tmp.i = (rands()[0]->evaluate(dbls, bools, sample).i || r);
    record (tmp, sample);
  }
  else 
  {
    tmp.i = (rands()[0]->evaluate(dbls, bools, sample).i ||
	     rands()[1]->evaluate(dbls, bools, sample).i);
  }

  return tmp;
}

genetic_expr* 
expr_or::clone ()
{
  expr_or *tmp = new expr_or (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_not::expr_not (genetic_expr_bnf* bnf, 
		    genetic_expr* parent) :
///////////////////////////////////////////
//////////////// !expr ////////////////////
  boolean_expr (bnf, parent, 1)
{
}

genetic_expr*
expr_not::random (genetic_expr_bnf* bnf, 
		  uint32 h,
		  genetic_expr* parent)
{
  expr_not *_not = new expr_not (bnf, parent);
  _not->rands()[0] = genetic_expr::random (bnf, h, _not, BOOLEAN);
  return _not;
}

genetic_expr*
expr_not::parse_expr (genetic_expr_bnf* bnf, 
		      genetic_expr* parent, 
		      genetic_lexer* lexer)
{
  expr_not *_not = new expr_not (bnf, parent);
  _not->rands()[0] = genetic_expr::parse_bool_expr (bnf, _not, lexer);
  return _not;
}

lval
expr_not::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;

  tmp.i = !(rands()[0]->evaluate(dbls, bools, sample).i);
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_not::clone ()
{
  expr_not *tmp = new expr_not (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_bool_const::expr_bool_const (genetic_expr_bnf* bnf, 
				  genetic_expr* parent, 
				  bool k) :
///////////////////////////////////////////
/////////// boolean constant //////////////
  boolean_expr (bnf, parent, 0),
  m_const(k)
{
}

genetic_expr*
expr_bool_const::random (genetic_expr_bnf* bnf, 
			 genetic_expr* parent)
{
  uint32 tmp = ::random() % 16;
  return new expr_bool_const (bnf, parent, tmp&1);
}

genetic_expr*
expr_bool_const::parse_expr (genetic_expr_bnf* bnf, 
			     genetic_expr* parent,
			     genetic_lexer* lexer)
{
  if (lexer->token() != BOOLVAL)
    lexer->error ("Expecting a boolean value");
  return new expr_bool_const (bnf, parent, lexer->bool_val());
}

lval
expr_bool_const::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;

  tmp.i = m_const;
  record (tmp, sample);
  return tmp;
}

genetic_expr*
expr_bool_const::clone () 
{
  return new expr_bool_const (m_bnf, NULL, m_const);
}

void
expr_bool_const::print (ostream& s,
			bool state) const
{
  print_prologue (s, state);
  s << k_expr_bool_const;
  if (m_const) 
    s << " true";
  else 
    s << " false";
  print_epilogue (s, state);
}

void
expr_bool_const::mutate_node ()
{
  m_const = !m_const;
}

expr_bool_arg::expr_bool_arg (genetic_expr_bnf* bnf, 
			      genetic_expr* parent, 
			      uint32 argnum) :
///////////////////////////////////////////
/////////// boolean argument //////////////
  boolean_expr (bnf, parent, 0),
  m_argnum (argnum)
{
}

genetic_expr*
expr_bool_arg::random (genetic_expr_bnf* bnf, 
		       genetic_expr* parent)
{
  uint32 tmp = ::random() % bnf->bargs();
  return new expr_bool_arg (bnf, parent, tmp);
}

genetic_expr*
expr_bool_arg::parse_expr (genetic_expr_bnf* bnf, 
			   genetic_expr* parent,
			   genetic_lexer* lexer)
{
  if (lexer->token() != BOOLARG)
    lexer->error ("Expecting a boolean argument");
  return new expr_bool_arg (bnf, parent, lexer->arg_num());
}

lval
expr_bool_arg::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  tmp.i = bools[m_argnum];
  record (tmp, sample);

  return tmp;
}

genetic_expr*
expr_bool_arg::clone () 
{
  return new expr_bool_arg (m_bnf, NULL, m_argnum);
}

void
expr_bool_arg::print (ostream& s, 
		      bool state) const
{
  print_prologue (s, state);
  s << k_expr_bool_arg << " $b" << m_argnum;
  print_epilogue (s, state);
}

void
expr_bool_arg::mutate_node ()
{
  // Choose another node.
  m_argnum = ::random() % m_bnf->bargs();
}
