/************************************************************************
 *
 *   File: boolean_expr.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Aug  6 12:15:12 2002
 *
 *   Function: Expressions that return boolean values.
 *
 ************************************************************************/

#ifndef __BOOLEAN_EXPR_H__
#define __BOOLEAN_EXPR_H__


#include "genetic_expr.h"


///////////////////////////////////////////
////////// boolean expression /////////////

class boolean_expr : public genetic_expr
{
 protected:
  boolean_expr (genetic_expr_bnf*, genetic_expr*, uint32);
  uint32 typing () const  { return BOOLEAN; }
};


///////////////////////////////////////////
/////////////// expr < expr ///////////////

class expr_lt : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_lt () {};

protected:
  expr_lt (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_lt; }
};


///////////////////////////////////////////
/////////////// expr > expr ///////////////

class expr_gt : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_gt () {};

protected:
  expr_gt (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_gt; }
};


///////////////////////////////////////////
////////////// expr == expr ///////////////

class expr_eq : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*,
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_eq () {};

protected:
  expr_eq (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_eq; }
};


///////////////////////////////////////////
////////////// expr && expr ///////////////

class expr_and : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_and () {};

protected:
  expr_and (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_and; }
};


///////////////////////////////////////////
////////////// expr || expr ///////////////

class expr_or : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_or () {};

protected:
  expr_or (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_or; }
};


///////////////////////////////////////////
////////////////// !expr //////////////////

class expr_not : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_not () {};

protected:
  expr_not (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_not; }
};


///////////////////////////////////////////
/////////// boolean constant //////////////

class expr_bool_const : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  void print (ostream&, bool = false) const;
  ~expr_bool_const () {}

 protected:
  expr_bool_const (genetic_expr_bnf*, genetic_expr*, bool k);
  char* opcode () const      { return k_expr_bool_const; }
  void  mutate_node ();
  bool  is_constant () const { return true; }

 private:
  bool m_const;
};


///////////////////////////////////////////
/////////// boolean argument //////////////

class expr_bool_arg : public boolean_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr* clone ();
  void print (ostream&, bool = false) const;
  ~expr_bool_arg () {}

 protected:
  expr_bool_arg (genetic_expr_bnf*, genetic_expr*, uint32 k);
  char* opcode () const    { return k_expr_bool_arg; }
  void mutate_node ();

 private:
  uint32 m_argnum;
};


#endif
