/************************************************************************
 *
 *   File: double_expr.cc:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Aug  6 12:17:30 2002
 *
 *   Function: Expressions that return a double precision value.
 *
 ************************************************************************/


#include "double_expr.h"
#include <math.h>

double_expr::double_expr (genetic_expr_bnf* bnf, 
			  genetic_expr* parent,
			  uint32 num_rands) :
///////////////////////////////////////////
///////// double_expr stuff. //////////////
  genetic_expr (bnf, parent, num_rands)
{
}


expr_add::expr_add (genetic_expr_bnf* bnf, genetic_expr* parent) :
///////////////////////////////////////////
////////////// expr + expr ////////////////
  double_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_add::random (genetic_expr_bnf* bnf, 
		  uint32 h,
		  genetic_expr* parent)
{
  expr_add *add;
  
  add = new expr_add (bnf, parent);
  add->rands()[0] = genetic_expr::random (bnf, h, add);
  add->rands()[1] = genetic_expr::random (bnf, h, add);
  return add;
}

genetic_expr*
expr_add::parse_expr (genetic_expr_bnf *bnf, 
		      genetic_expr *parent,
		      genetic_lexer *lexer)
{
  expr_add *add = new expr_add (bnf, parent);
  add->rands()[0] = genetic_expr::parse_dbl_expr (bnf, add, lexer);
  add->rands()[1] = genetic_expr::parse_dbl_expr (bnf, add, lexer);
  return add;  
}

lval
expr_add::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  
  tmp.d = rands()[0]->evaluate(dbls, bools, sample).d + 
    rands()[1]->evaluate(dbls, bools, sample).d;
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_add::clone ()
{
  expr_add *tmp = new expr_add (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_sub::expr_sub (genetic_expr_bnf* bnf, genetic_expr* parent) :
///////////////////////////////////////////
////////////// expr - expr ////////////////
  double_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_sub::random (genetic_expr_bnf* bnf, 
		  uint32 h,
		  genetic_expr* parent)
{
  expr_sub *sub = new expr_sub (bnf, parent);
  sub->rands()[0] = genetic_expr::random (bnf, h, sub);
  sub->rands()[1] = genetic_expr::random (bnf, h, sub);
  return sub;
}

genetic_expr*
expr_sub::parse_expr (genetic_expr_bnf *bnf, 
		      genetic_expr *parent,
		      genetic_lexer *lexer)
{
  expr_sub *sub = new expr_sub (bnf, parent);
  sub->rands()[0] = genetic_expr::parse_dbl_expr (bnf, sub, lexer);
  sub->rands()[1] = genetic_expr::parse_dbl_expr (bnf, sub, lexer);
  return sub;  
}

lval
expr_sub::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  
  tmp.d = rands()[0]->evaluate(dbls, bools, sample).d - 
    rands()[1]->evaluate(dbls, bools, sample).d;
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_sub::clone ()
{
  expr_sub *tmp = new expr_sub (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_mul::expr_mul (genetic_expr_bnf* bnf, genetic_expr* parent) :
///////////////////////////////////////////
////////////// expr * expr ////////////////
  double_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_mul::random (genetic_expr_bnf* bnf, 
		  uint32 h,
		  genetic_expr* parent)
{
  expr_mul *mul = new expr_mul (bnf, parent);
  mul->rands()[0] = genetic_expr::random (bnf, h, mul);
  mul->rands()[1] = genetic_expr::random (bnf, h, mul);
  return mul;
}

genetic_expr*
expr_mul::parse_expr (genetic_expr_bnf *bnf, 
		      genetic_expr *parent,
		      genetic_lexer *lexer)
{
  expr_mul *mul = new expr_mul (bnf, parent);
  mul->rands()[0] = genetic_expr::parse_dbl_expr (bnf, mul, lexer);
  mul->rands()[1] = genetic_expr::parse_dbl_expr (bnf, mul, lexer);
  return mul;  
}

lval
expr_mul::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  
  tmp.d = rands()[0]->evaluate(dbls, bools, sample).d * 
    rands()[1]->evaluate(dbls, bools, sample).d;
  record (tmp, sample);

  return tmp;
}

genetic_expr* 
expr_mul::clone ()
{
  expr_mul *tmp = new expr_mul (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_div::expr_div (genetic_expr_bnf* bnf, 
		    genetic_expr* parent) :
///////////////////////////////////////////
////////////// expr / expr ////////////////
  double_expr (bnf, parent, 2)
{
}

genetic_expr*
expr_div::random (genetic_expr_bnf* bnf, 
		  uint32 h,
		  genetic_expr* parent)
{
  expr_div *div = new expr_div (bnf, parent);
  div->rands()[0] = genetic_expr::random (bnf, h, div);
  div->rands()[1] = genetic_expr::random (bnf, h, div);
  return div;
}

genetic_expr*
expr_div::parse_expr (genetic_expr_bnf *bnf, 
		      genetic_expr *parent,
		      genetic_lexer *lexer)
{
  expr_div *div = new expr_div (bnf, parent);
  div->rands()[0] = genetic_expr::parse_dbl_expr (bnf, div, lexer);
  div->rands()[1] = genetic_expr::parse_dbl_expr (bnf, div, lexer);
  return div;  
}

lval
expr_div::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  double denom;
  double num;

  num = rands()[0]->evaluate(dbls, bools, sample).d;
  denom = rands()[1]->evaluate(dbls, bools, sample).d;
  tmp.d = (denom == 0.0) ? 0.0 : (num / denom);
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_div::clone ()
{
  expr_div *tmp = new expr_div (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_sqrt::expr_sqrt (genetic_expr_bnf* bnf, 
		      genetic_expr* parent) :
///////////////////////////////////////////
////////////// sqrt(expr) /////////////////
  double_expr (bnf, parent, 1)
{
}

genetic_expr*
expr_sqrt::random (genetic_expr_bnf* bnf, 
		   uint32 h,
		   genetic_expr* parent)
{
  expr_sqrt *tmp = new expr_sqrt (bnf, parent);
  tmp->rands()[0] = genetic_expr::random (bnf, h, tmp);
  return tmp;
}

genetic_expr*
expr_sqrt::parse_expr (genetic_expr_bnf *bnf, 
		       genetic_expr *parent,
		       genetic_lexer *lexer)
{
  expr_sqrt *tmp = new expr_sqrt (bnf, parent);
  tmp->rands()[0] = genetic_expr::parse_dbl_expr (bnf, tmp, lexer);
  return tmp;  
}

lval
expr_sqrt::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  double rand;

  rand = rands()[0]->evaluate(dbls, bools, sample).d;
  if (rand < 0) rand = -rand;
  tmp.d = sqrt(rand);
  record (tmp, sample);
  return tmp;
}

genetic_expr* 
expr_sqrt::clone ()
{
  expr_sqrt *tmp = new expr_sqrt (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_ternary::expr_ternary (genetic_expr_bnf* bnf, 
			    genetic_expr* parent) :
///////////////////////////////////////////
////////// expr ? expr : expr /////////////
  double_expr (bnf, parent, 3)
{
}

genetic_expr*
expr_ternary::random (genetic_expr_bnf* bnf, 
		      uint32 h,
		      genetic_expr* parent)
{
  expr_ternary *tern = new expr_ternary (bnf, parent);
  tern->rands()[0] = genetic_expr::random (bnf, h, tern, BOOLEAN);
  tern->rands()[1] = genetic_expr::random (bnf, h, tern);
  tern->rands()[2] = genetic_expr::random (bnf, h, tern);
  return tern;
}

genetic_expr*
expr_ternary::parse_expr (genetic_expr_bnf *bnf, 
			  genetic_expr *parent,
			  genetic_lexer *lexer)
{
  expr_ternary *tern = new expr_ternary (bnf, parent);
  tern->rands()[0] = genetic_expr::parse_bool_expr (bnf, tern, lexer);
  tern->rands()[1] = genetic_expr::parse_dbl_expr (bnf, tern, lexer);
  tern->rands()[2] = genetic_expr::parse_dbl_expr (bnf, tern, lexer);
  return tern;  
}

lval
expr_ternary::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  int32 r0;

  r0 = rands()[0]->evaluate(dbls, bools, sample).i;
  if (sample)
  {
    double r1 = rands()[1]->evaluate(dbls, bools, sample).d;
    double r2 = rands()[2]->evaluate(dbls, bools, sample).d;
    tmp.d = r0 ? r1 : r2;
    record (tmp, sample);    
  }
  else
  {
    tmp.d = r0 ?
      rands()[1]->evaluate(dbls, bools, sample).d :
      rands()[2]->evaluate(dbls, bools, sample).d;
  }

  return tmp;
}

genetic_expr* 
expr_ternary::clone ()
{
  expr_ternary *tmp = new expr_ternary (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_cmul::expr_cmul (genetic_expr_bnf* bnf, 
		      genetic_expr* parent) :
///////////////////////////////////////////
///// expr0 ? expr1 * expr2 : expr2 //////
  double_expr (bnf, parent, 3)
{
}

genetic_expr*
expr_cmul::random (genetic_expr_bnf* bnf, 
		   uint32 h,
		   genetic_expr* parent)
{
  expr_cmul *cm = new expr_cmul (bnf, parent);
  cm->rands()[0] = genetic_expr::random (bnf, h, cm, BOOLEAN);
  cm->rands()[1] = genetic_expr::random (bnf, h, cm);
  cm->rands()[2] = genetic_expr::random (bnf, h, cm);
  return cm;
}

genetic_expr*
expr_cmul::parse_expr (genetic_expr_bnf *bnf, 
		       genetic_expr *parent,
		       genetic_lexer *lexer)
{
  expr_cmul *cm = new expr_cmul (bnf, parent);
  cm->rands()[0] = genetic_expr::parse_bool_expr (bnf, cm, lexer);
  cm->rands()[1] = genetic_expr::parse_dbl_expr (bnf, cm, lexer);
  cm->rands()[2] = genetic_expr::parse_dbl_expr (bnf, cm, lexer);
  return cm;  
}

lval
expr_cmul::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  double r2;
  int32 r0;

  r0 = rands()[0]->evaluate(dbls, bools, sample).i;
  r2 = rands()[2]->evaluate(dbls, bools, sample).d;
  if (sample)
  {
    double r1 = rands()[1]->evaluate(dbls, bools, sample).d;
    tmp.d = r0 ? r1 * r2 : r2;
    record (tmp, sample);
  }
  else
  {
    tmp.d = r0 ? 
      rands()[1]->evaluate(dbls, bools, sample).d * r2 : r2;
  }

  return tmp;
}

genetic_expr* 
expr_cmul::clone ()
{
  expr_cmul *tmp = new expr_cmul (m_bnf, NULL);
  tmp->clone_rands (this);
  return tmp;
}

expr_dbl_const::expr_dbl_const (genetic_expr_bnf* bnf, 
				genetic_expr* parent, 
				double k) :
///////////////////////////////////////////
//////////// double constant //////////////
  double_expr (bnf, parent, 0),
  m_const(k)
{
}

genetic_expr*
expr_dbl_const::random (genetic_expr_bnf* bnf, 
			genetic_expr* parent)
{
  double tmp = 10.0 * drand48 ();
  return new expr_dbl_const (bnf, parent, tmp);
}

genetic_expr*
expr_dbl_const::parse_expr (genetic_expr_bnf *bnf, 
			    genetic_expr *parent,
			    genetic_lexer *lexer)
{
  if (lexer->token() != DBLVAL)
    lexer->error ("Expecting a double precision value");
  return new expr_dbl_const (bnf, parent, lexer->double_val());
}

lval
expr_dbl_const::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;

  tmp.d = m_const;
  record (tmp, sample);
  return tmp;
}

genetic_expr*
expr_dbl_const::clone () 
{
  return new expr_dbl_const (m_bnf, NULL, m_const);
}

void
expr_dbl_const::print (ostream& os, 
		       bool state) const
{
  uint32 old_precision = os.precision ();

  print_prologue (os, state);
  os.setf (ios::fixed, ios::floatfield);
  os.precision (4);
  os << k_expr_dbl_const << " " << m_const;
  os.precision (old_precision);
  print_epilogue (os, state);
}

double 
expr_dbl_const::gaussian (double mean, double standard_deviation) const
{
  double v1;
  double v2;
  double norm_squared;
  
  // Find a random point in the unit circle.
  do {
    v1 = 2 * drand48() - 1;
    v2 = 2 * drand48() - 1;
    norm_squared = v1*v1 + v2*v2;
  } while (norm_squared > 1 || norm_squared == 0);
  
  double factor = sqrt(-2 * log(norm_squared) / norm_squared);
  return v2 * factor * standard_deviation + mean;
}

void
expr_dbl_const::mutate_node ()
{
  if (::random() & 0x1)
    // Perturb the constant slightly.
    m_const = m_const * gaussian (2.0, 0.25);
  else
    m_const = 10 * drand48 ();
}

expr_dbl_arg::expr_dbl_arg (genetic_expr_bnf* bnf, 
			    genetic_expr* parent, 
			    uint32 argnum) :
///////////////////////////////////////////
//////////// double argument //////////////
  double_expr (bnf, parent, 0),
  m_argnum (argnum)
{
}

genetic_expr*
expr_dbl_arg::random (genetic_expr_bnf* bnf, genetic_expr* parent)
{
  uint32 tmp = ::random() % bnf->dargs();
  return new expr_dbl_arg (bnf, parent, tmp);
}

genetic_expr*
expr_dbl_arg::parse_expr (genetic_expr_bnf *bnf, 
			  genetic_expr *parent,
			  genetic_lexer *lexer)
{
  if (lexer->token() != DBLARG)
    lexer->error ("Expecting a double precision argument");
  return new expr_dbl_arg (bnf, parent, lexer->arg_num());
}

lval
expr_dbl_arg::evaluate (double dbls[], int32 bools[], bool sample)
{
  lval tmp;
  tmp.d = dbls[m_argnum];
  record (tmp, sample);
  return tmp;
}

genetic_expr*
expr_dbl_arg::clone () 
{
  return new expr_dbl_arg (m_bnf, NULL, m_argnum);
}

void
expr_dbl_arg::print (ostream& s, 
		     bool state) const
{
  print_prologue (s, state);
  s << k_expr_dbl_arg << " $d" << m_argnum;
  print_epilogue (s, state);
}

void
expr_dbl_arg::mutate_node ()
{
  // Choose another node.
  m_argnum = ::random() % m_bnf->dargs();
}
