/*************************************************************************
 *
 *   File: compiler_utils.cc:
 *   
 *   Author: Mark Stephenson
 *   Date: Wed Jul 17 11:52:18 2003
 *
 *   Function: A few functions that handle compiling and running
 *             of the benchmarks.
 *
 ************************************************************************/

#include "exec_utils.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <math.h>
#include <bits/nan.h>
#include <errno.h>

// A global flag that specifies verbosity.
bool g_verbose = false;

void
execute_commands (command_list *cmds,
		  bool ignore)
/////////////////////////////////////////////////////////////
//  
//  Executes the commands in the command list.
//
/////////////////////////////////////////////////////////////
{
  command_list *beg;
  int32 sysval;

  beg = cmds;
  while (beg != NULL)
  {
    string command = beg->entry;
    if (g_verbose) 
    {
      cerr << "FINCH: executing " << command << endl;
    }
    else
    {
      command += string(" >& /dev/null");      
    }
    sysval = system (command.c_str());
    if (!ignore && sysval != 0) 
    {
      cerr << "System call returned nonzero:\n";
      cerr << beg->entry << endl;
    }
    beg = beg->next;
  }
}


bool
finch_clean_dir (const string& name,
		 int(*select)(const struct dirent *))
/////////////////////////////////////////////////////////////
//  
//  Cleans the directory with name name.
//
/////////////////////////////////////////////////////////////
{
  int numpfiles;
  struct dirent** pfilelist;
  string nlevel;
  bool passed;

  numpfiles = scandir (name.c_str(),
		       &pfilelist, 
		       select, 0);
    
  if (numpfiles == -1) return false;

  passed = true;
  for (int i = 0; i < numpfiles; i++)
  {
    if (strcmp(".", pfilelist[i]->d_name) != 0 &&
	strcmp("..", pfilelist[i]->d_name) != 0)
    {
      nlevel = name + string("/");
      nlevel += pfilelist[i]->d_name;
      passed &= finch_unlink (nlevel, select);
      free (pfilelist[i]);
    }
  }

  free (pfilelist);
  return passed;
}


bool
finch_unlink (const string& name, 
	      int(*select)(const struct dirent *))
/////////////////////////////////////////////////////////////
//  
//  Unlinks files and recursively unlinks directories.
//
/////////////////////////////////////////////////////////////
{
  struct stat buf;
  bool passed;

  if (lstat(name.c_str(), &buf) == -1)
    return false;

  passed = true;
  if (S_ISREG(buf.st_mode) || S_ISLNK(buf.st_mode))
  {
    passed = (unlink(name.c_str()) != -1);
  }
  else if (S_ISDIR(buf.st_mode))
  {
    passed = finch_clean_dir (name, select);
    passed &= (rmdir(name.c_str()) != -1);
  }
  else
  {
    passed = false;
  }
  return passed;
}


void
finch_sigint (int sig)
/////////////////////////////////////////////////////////////
//  
//  Catches C-c events and kills Finch.
//
/////////////////////////////////////////////////////////////
{
  cerr << "\nC-c pressed...Exiting Finch.\n";
  exit(0);
}
