/************************************************************************
 *
 *   File: pbs_harness.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Wed Aug 21 19:52:38 2002
 *
 *   Function: This harness uses a job queuing system to distribute
 *             jobs.  It uses the file system to collect fitnesses,
 *             and therefore relies on a networked file system (that
 *             ensures consistency).  It also relies on PBS, the 
 *             job queuing system we use.
 *
 ***********************************************************************/

#ifndef __PBS_HARNESS_H__
#define __PBS_HARNESS_H__


#include "genetic_expr.h"
#include "benchmark_set.h"
#include "compile_harness.h"


class pbs_harness : public compile_harness
{
 public:

  // Create a new harness.
  pbs_harness (uint32  population_size,
	       uint32  generations,
	       double  mutate_rate,
	       double  mortality,
	       char*   initial_pop_fname,
	       char*   save_pop_fname,
	       char*   finch_home,
	       uint32  tournament_size,
	       uint32  dss_set_size,
	       uint32  maximum_genome_size,
	       uint32  maximum_height,
	       uint32  elitist,
	       uint32  sunset,
	       bool    quantize_results,
	       bool    classifier,
	       benchmark_set *benches);
  
  // Kill the harness.
  ~pbs_harness ();

 protected:

  // Helper function that runs the benchmarks.
  void create_pbs_script (benchmark_info*, genetic_expr*, uint32);

  // Collect the fitnesses and cleanup directories.
  bool gather_results (benchmark_info*, genetic_expr*, uint32);

  // Writes the commands to the given stream.
  void write_commands (ofstream&, command_list*);

  // Submits the script to PBS.
  void submit_script (const string& name);

  // Clears the evaluate directory that we'll be
  // running in.
  virtual void clear_evaluate_dir ();

  //  Define a function that farms out the jobs and 
  //  collects results.
  virtual void farm_out_jobs ();

  void create_attr_list ();

  struct attropl *pbs_attr_push_front (struct attropl*, char*, char*);

 protected:

  int32 m_pbs_stream;
  struct attropl *m_common_attrs;
};

#endif
