/************************************************************************
 *
 *   File: simple_harness.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Wed Aug 21 19:52:38 2002
 *
 *   Function: This harness uses the file system to communicate
 *     expressions and fitnesses.  It's called simple_harness because
 *     it is very simple.  It uses a directory (set according to an
 *     environment variable) to communicate all information via files.
 *
 ***********************************************************************/

#ifndef __SIMPLE_HARNESS_H__
#define __SIMPLE_HARNESS_H__


#include "genetic_expr.h"
#include "benchmark_set.h"
#include "compile_harness.h"


class simple_harness : public compile_harness
{
 public:

  // Create a new harness.
  simple_harness (uint32  population_size,
		  uint32  generations,
		  double  mutate_rate,
		  double  mortality,
		  char*   initial_pop_fname,
		  char*   save_pop_fname,
		  char*   finch_home,
		  uint32  tournament_size,
		  uint32  dss_set_size,
		  uint32  maximum_genome_size,
		  uint32  maximum_height,
		  uint32  elitist,
		  uint32  sunset,
		  bool    quantize_results,
		  bool    classifier,
		  benchmark_set *benches);

  
  // Kill the harness.
  ~simple_harness ();

 protected:

  // Helper function that runs the benchmarks.
  void dispatch (benchmark_info*, genetic_expr*);

  // Clears the evaluate directory that we'll be
  // running in.
  virtual void clear_evaluate_dir ();

  //  Define a function that farms out the jobs and 
  //  collects results.
  virtual void farm_out_jobs ();
};

#endif
