/*
 * finch_jni.c: Java Native Interface glue for finch
 * David Maze <dmaze@cag.lcs.mit.edu>
 * $Id: edu_mit_lcs_cag_metaopt_Finch.c,v 1.3 2003/06/05 20:52:13 dmaze Exp $
 */

#include "edu_mit_lcs_cag_metaopt_Finch.h"
#include "finch_interface.h"
#include <stdlib.h>

void Java_edu_mit_lcs_cag_metaopt_Finch_initializeLib(JNIEnv *env,
                                                      jclass clazz)
{
  FINCH_initialize_lib();
}

jdouble Java_edu_mit_lcs_cag_metaopt_Finch_evaluateReal(JNIEnv *env,
                                                        jclass clazz,
                                                        jarray dbls_in,
                                                        jarray bools_in)
{
  /* No particular requirements on either array.  We do want the
   * counts and values. */
  unsigned num_dbls, num_bools;
  double *dbls;
  int *bools = NULL;
  jboolean *jbools;
  int i;
  double result = 0.0;
  
  num_dbls = (*env)->GetArrayLength(env, dbls_in);
  num_bools = (*env)->GetArrayLength(env, bools_in);
  dbls = (*env)->GetDoubleArrayElements(env, dbls_in, NULL);
  jbools = (*env)->GetBooleanArrayElements(env, bools_in, NULL);

  /* Exception handling: the JNI spec says that, if something fails,
   * it generally both throws an exception in Java space and returns
   * NULL to C space.  The actual JNI documentation seems a little
   * inconsistent as to whether or not we get an OutOfMemoryError
   * if Get*ArrayElements() fails, but we're thoroughly hosed if
   * it does. */
  if (dbls && jbools)
  {
    /* Need to make a copy, since jboolean is an unsigned char but
     * finch deals with booleans as int32. */
    bools = malloc(sizeof(int) * num_bools);
    if (!bools)
      (*env)->FatalError(env, "malloc returned NULL"); /* does not return */
    for (i = 0; i < num_bools; i++)
      bools[i] = jbools[i];
  
    /* Do it. */
    result = FINCH_evaluate_real_array(num_dbls, num_bools, dbls, bools);
  }
  
  /* Clean up. */
  if (bools) free(bools);
  if (jbools) (*env)->ReleaseBooleanArrayElements(env, bools_in, jbools, 0);
  if (dbls) (*env)->ReleaseDoubleArrayElements(env, dbls_in, dbls, 0);
  
  return result;
}

jboolean Java_edu_mit_lcs_cag_metaopt_Finch_evaluateBinary(JNIEnv *env,
                                                           jclass clazz,
                                                           jarray dbls_in,
                                                           jarray bools_in)
{
  /* No particular requirements on either array.  We do want the
   * counts and values. */
  unsigned num_dbls, num_bools;
  double *dbls;
  int *bools = NULL;
  jboolean *jbools;
  int i;
  int result;
  
  num_dbls = (*env)->GetArrayLength(env, dbls_in);
  num_bools = (*env)->GetArrayLength(env, bools_in);
  dbls = (*env)->GetDoubleArrayElements(env, dbls_in, NULL);
  jbools = (*env)->GetBooleanArrayElements(env, bools_in, NULL);
  
  /* Exception handling: the JNI spec says that, if something fails,
   * it generally both throws an exception in Java space and returns
   * NULL to C space.  The actual JNI documentation seems a little
   * inconsistent as to whether or not we get an OutOfMemoryError
   * if Get*ArrayElements() fails, but we're thoroughly hosed if
   * it does. */
  if (dbls && jbools)
  {
    /* Need to make a copy, since jboolean is an unsigned char but
     * finch deals with booleans as int32. */
    bools = malloc(sizeof(int) * num_bools);
    if (!bools)
      (*env)->FatalError(env, "malloc returned NULL"); /* does not return */
    for (i = 0; i < num_bools; i++)
      bools[i] = jbools[i];
  
    /* Do it. */
    result = FINCH_evaluate_real_array(num_dbls, num_bools, dbls, bools);
  }
  
  /* Clean up. */
  if (bools) free(bools);
  if (jbools) (*env)->ReleaseBooleanArrayElements(env, bools_in, jbools, 0);
  if (dbls) (*env)->ReleaseDoubleArrayElements(env, dbls_in, dbls, 0);
  
  return result;
}

void Java_edu_mit_lcs_cag_metaopt_Finch_finalizeLib(JNIEnv *env,
                                                    jclass clazz)
{
  FINCH_finalize_lib();
}
